"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSchemaFromSimpleSchema = void 0;
const internal_1 = require("@fluidframework/core-utils/internal");
const index_js_1 = require("../../util/index.js");
const index_js_2 = require("../core/index.js");
const schemaTypes_js_1 = require("../schemaTypes.js");
const schemaFactory_js_1 = require("./schemaFactory.js");
const factory = new schemaFactory_js_1.SchemaFactory(undefined);
/**
 * Create {@link FieldSchema} from a SimpleTreeSchema.
 * @remarks
 * Only use this API if hand written schema (produced using {@link SchemaFactory} cannot be provided.
 *
 * Using generated schema with schema aware APIs (designed to work with strongly typed schema) like {@link TreeViewConfiguration}
 * will produce a poor TypeScript typing experience which is subject to change.
 *
 * Editing through a view produced using this schema can easily violate invariants other users of the document might expect and must be done with great care.
 * @internal
 */
function generateSchemaFromSimpleSchema(simple) {
    const context = new Map([...simple.definitions].map(([id, schema]) => [
        id,
        // This relies on the caching in evaluateLazySchema so that it only runs once.
        () => generateNode(id, schema, context),
    ]));
    return generateFieldSchema(simple, context);
}
exports.generateSchemaFromSimpleSchema = generateSchemaFromSimpleSchema;
function generateFieldSchema(simple, context) {
    return (0, schemaTypes_js_1.createFieldSchema)(simple.kind, generateAllowedTypes(simple.allowedTypes, context));
}
function generateAllowedTypes(allowed, context) {
    return [...allowed].map((id) => context.get(id) ?? (0, index_js_1.fail)(0xb5a /* Missing schema */));
}
function generateNode(id, schema, context) {
    switch (schema.kind) {
        case index_js_2.NodeKind.Object: {
            const fields = {};
            for (const [key, field] of Object.entries(schema.fields)) {
                fields[key] = generateFieldSchema(field, context);
            }
            return factory.object(id, fields);
        }
        case index_js_2.NodeKind.Array:
            return factory.array(id, generateAllowedTypes(schema.allowedTypes, context));
        case index_js_2.NodeKind.Map:
            return factory.map(id, generateAllowedTypes(schema.allowedTypes, context));
        case index_js_2.NodeKind.Leaf:
            return (schemaFactory_js_1.SchemaFactory.leaves.find((leaf) => leaf.identifier === id) ??
                (0, index_js_1.fail)(0xb5b /* Missing schema */));
        default:
            return (0, internal_1.unreachableCase)(schema);
    }
}
//# sourceMappingURL=schemaFromSimple.js.map